<?php

namespace Teknisa\FIN\Modules\FluxoCaixa\Factory;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Sdoccorr;
use Teknisa\Model\Financeiro\Entities\Movfxajbolp;
use Teknisa\Model\Financeiro\Entities\Movfxajbolr;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;
use Teknisa\Model\Financeiro\Entities\Tipoajreceber;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Geral\Entities\Paramgeral;

/**
 * Class FluxoDeCaixaFactory
 * @package Teknisa\FIN\Modules\FluxoCaixa\Factory
 */
class FluxoDeCaixaFactory {

    protected $entityManager;
    protected $FluxoDeCaixaService;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        #initializes default service
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getParamgeral(){
        $parans = array();
        $paramgeral = $this->entityManager->getRepository(Paramgeral::class)->findOneBy($parans);
        return $paramgeral;
    }

    public function buscaOrcamentofinanceiro($nrorg, $cdempresa, $cdfilial, $cdclassfina, $primeiroDiaMes, $ultimoDiaMes){
        $sql = <<<SQL
            SELECT CDEMPRESA, CDFILIAL, CDCLASSFINA, DTORCAFINA,
                   VRORCAFINA, VREXTOLEORCA, VRREALORCA
              FROM ORCAFINA
             WHERE (CDEMPRESA   = :CDEMPRESA)               AND
                   (NRORG       = :NRORG)                   AND
                   (CDFILIAL    = NVL(:CDFILIAL, CDFILIAL)) AND
                   (CDCLASSFINA = :CDCLASSFINA)             AND
                   (DTORCAFINA  BETWEEN :DTINI AND :DTFIN)
SQL;
        $params = array(
            'NRORG' => $nrorg,
            'CDEMPRESA' => $cdempresa,
            'CDFILIAL' => $cdfilial,
            'CDCLASSFINA' => $cdclassfina,
            'DTINI' => $primeiroDiaMes,
            'DTFIN' => $ultimoDiaMes
        );
        return $this->entityManager->getConnection()->fetchColumn($sql, $params);
    }

    /**
     * Busca Movimentação de Fluxo
     * @param $nrorg
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @return Movfluxo
     */
    public function buscaMovFluxo($nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo){
        $params = array();
        $params['nrorg'] = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['cdfilial'] = $cdfilial;
        $params['cdclassfina'] = $cdclassfina;
        $params['dtmovfluxo'] = $dtmovfluxo;
        $params['idmovfluxo'] = $idmovfluxo;
        $movfluxo = $this->entityManager->getRepository(Movfluxo::class)->findOneBy($params);
        return $movfluxo;
    }

    /**
     * Retorna todos os saldos com data superior
     * à data da movimentação bancária.
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $nrorg
     * @return Sdoccorr
     */
    public function buscaSaldoContCorr($cdreducont, $dtmovbanc, $nrorg){
        /** @var Sdoccorr $saldos */
        $saldos = Sdoccorr::getObjectsByWhere('CDREDUCONT = ? AND DTSDOCCORR >= ? AND NRORG = ? ORDER BY DTSDOCCORR',
            array($cdreducont, $dtmovbanc, $nrorg));
        return $saldos;
    }

    /**
     * Insere Movimentação de Fluxo
     * @param $nrorg
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdclassfina
     * @param $dtmovfluxo
     * @param $idmovfluxo
     * @param $vrmovfluxo
     * @param $idoperacao
     * @return Movfluxo
     */
    public function insereMovFluxo($nrorg, $cdempresa, $cdfilial, $cdclassfina, $dtmovfluxo, $idmovfluxo, $vrmovfluxo, $idoperacao){
        /** @var Movfluxo $movfluxo */
        $movfluxo = new Movfluxo();
        $movfluxo->setNrorg($nrorg);
        $movfluxo->setCdfilial($cdfilial);
        $movfluxo->setCdempresa($cdempresa);
        $movfluxo->setCdclassfina($cdclassfina);
        $movfluxo->setDtmovfluxo($dtmovfluxo);
        $movfluxo->setIdmovfluxo($idmovfluxo);
        $movfluxo->setVrentmfluxo($idoperacao == 'E'?$vrmovfluxo:0);
        $movfluxo->setVrsaimfluxo($idoperacao == 'S'?$vrmovfluxo:0);
        $movfluxo->setDtinclusao(DateUtil::getDataAtual());
        $movfluxo->setDtultatu(DateUtil::getDataAtual());
        return $movfluxo;
    }

    /**
     * Atualiza Movimentação de Fluxo
     * @param Movfluxo $movfluxo
     * @param $vrmovfluxo
     * @param $idoperacao
     * @return Movfluxo
     */
    public function atualizaMovFluxo(Movfluxo $movfluxo, $vrmovfluxo, $idoperacao){
        // Verifica Operação Entrada/Saida
        if($idoperacao == 'E')
            $movfluxo->setVrentmfluxo($movfluxo->getVrentmfluxo() + $vrmovfluxo);
        else $movfluxo->setVrsaimfluxo($movfluxo->getVrsaimfluxo() + $vrmovfluxo);
        $movfluxo->setDtultatu(DateUtil::getDataAtual());

        return $movfluxo;
    }

    public function insereSaldoContaCorrente($nrorg, $cdreducont, $dtmovbanc, $saldo){
        $novosaldo = new Sdoccorr();
        $novosaldo->setNrorg($nrorg);
        $novosaldo->setDtinclusao(DateUtil::getDataAtual());
        $novosaldo->setCdreducont($cdreducont);
        $novosaldo->setDtsdoccorr($dtmovbanc);
        $novosaldo->setVrsdoccorr($saldo);

        return $novosaldo;
    }

    /**
     * Retorna saldo anterior à data de Movimento
     * @param $nrorg
     * @param $cdreducont
     * @param $dtmovbanc
     * @return mixed
     */
    public function getSaldoAnterior($nrorg, $cdreducont, $dtmovbanc){
        $sql = <<<SQL
        SELECT VRSDOCCORR
        FROM SDOCCORR
        WHERE CDREDUCONT = :P_CDREDUCONT  AND
              NRORG = :P_NRORG AND
              DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                            FROM SDOCCORR
                            WHERE CDREDUCONT = :P_CDREDUCONT AND
                                  DTSDOCCORR <= :P_DATA AND
                                  NRORG = :P_NRORG)
SQL;
        $params = array(
            'P_NRORG' => $nrorg,
            'P_CDREDUCONT' => $cdreducont,
            'P_DATA' => $dtmovbanc
        );
        return $this->entityManager->getConnection()->fetchColumn($sql, $params);
    }
}